@@ -258,12 +258,6 @@ const ChatView = new Lang.Class({
 
         this.connect('screen-changed',
                      Lang.bind(this, this._updateIndent));
-        this.connect('map', Lang.bind(this, this._updateActive));
-        this.connect('unmap', Lang.bind(this, this._updateActive));
-        this.connect('parent-set',
-                     Lang.bind(this, this._updateToplevel));
-        this.connect('state-flags-changed',
-                     Lang.bind(this, this._updateToplevel));
         this.connect('scroll-event', Lang.bind(this, this._onScroll));
         this.connect('edge-reached', (w, pos) => {
             if (pos == Gtk.PositionType.BOTTOM)
@@ -290,8 +284,6 @@ const ChatView = new Lang.Class({
 
         this._room = room;
         this._state = { lastNick: null, lastTimestamp: 0, lastStatusGroup: 0 };
-        this._active = false;
-        this._toplevelFocus = false;
         this._joinTime = 0;
         this._maxNickChars = MAX_NICK_CHARS;
         this._hoveredButtonTags = [];
@@ -325,6 +317,9 @@ const ChatView = new Lang.Class({
         this._app = Gio.Application.get_default();
         PasteManager.DropTargetIface.addTargets(this, this._view);
 
+        this._app.connect('room-focus-changed',
+                          Lang.bind(this, this._checkMessages));
+
         this._hoverCursor = Gdk.Cursor.new(Gdk.CursorType.HAND1);
 
         this._channelSignals = [];
@@ -603,23 +598,6 @@ const ChatView = new Lang.Class({
         this._view.left_margin = MARGIN + totalWidth;
     },
 
-    _updateActive: function() {
-        let active = this.get_mapped();
-        if (this._active == active)
-            return;
-        this._active = active;
-        this._checkMessages();
-    },
-
-    _updateToplevel: function() {
-        let flags = this.get_state_flags();
-        let toplevelFocus = !(flags & Gtk.StateFlags.BACKDROP);
-        if (this._toplevelFocus == toplevelFocus)
-            return;
-        this._toplevelFocus = toplevelFocus;
-        this._checkMessages();
-    },
-
     _updateScroll: function() {
         if (!this._autoscroll)
             return;
@@ -708,15 +686,18 @@ const ChatView = new Lang.Class({
     },
 
     _pendingMessageRemoved: function(channel, message) {
-        let [id,] = message.get_pending_message_id();
-        let mark = this._pending.get(id);
-        if (!mark)
+        let [id, valid] = message.get_pending_message_id();
+        if (!valid || !this._pending.has(id))
             return;
+        this._removePendingMark(id);
+    },
+
+    _removePendingMark: function(id) {
+        let mark = this._pending.get(id);
         // Re-enable auto-scrolling if this is the most recent message
         if (this._view.buffer.get_iter_at_mark(mark).is_end())
             this._autoscroll = true;
         this._view.buffer.delete_mark(mark);
-        this._app.withdraw_notification('pending-message-' + id);
         this._pending.delete(id);
     },
 
@@ -826,7 +807,7 @@ const ChatView = new Lang.Class({
     },
 
     _checkMessages: function() {
-        if (!this._active || !this._toplevelFocus || !this._channel)
+        if (!this._app.isRoomFocused(this._room) || !this._channel)
             return;
 
         this._needsIndicator = true;
@@ -893,6 +874,11 @@ const ChatView = new Lang.Class({
         if (this._channel == this._room.channel)
             return;
 
+        // Pending IDs are invalidated by channel changes, so
+        // remove marks to not get stuck on highlighted messages
+        for (let id of this._pending.keys())
+            this._removePendingMark(id);
+
         if (this._channel) {
             for (let i = 0; i < this._channelSignals.length; i++)
                 this._channel.disconnect(this._channelSignals[i]);
@@ -1203,8 +1189,6 @@ const ChatView = new Lang.Class({
 
     _insertTpMessage: function(tpMessage) {
         let message = this._createMessage(tpMessage);
-        let shouldHighlight = this._room.should_highlight_message(message.nick,
-                                                                  message.text);
 
         this._ensureNewLine();
 
@@ -1212,25 +1196,8 @@ const ChatView = new Lang.Class({
         this._insertMessage(iter, message, this._state);
         this._trackContact(tpMessage.sender);
 
-        if (shouldHighlight &&
-            !(this._toplevelFocus && this._active)) {
-            let summary = '%s %s'.format(this._room.display_name, message.nick);
-            let notification = new Gio.Notification();
-            notification.set_title(summary);
-            notification.set_body(message.text);
-
-            let account = this._room.account;
-            let param = GLib.Variant.new('(ssu)',
-                                         [ account.get_object_path(),
-                                           this._room.channel_name,
-                                           Utils.getTpEventTime() ]);
-            notification.set_default_action_and_target('app.join-room', param);
-            this._app.send_notification('pending-message-' + message.pendingId,
-                                        notification);
-        }
-
-        if (!message.pendingId /* outgoing */ ||
-            (this._active && this._toplevelFocus && this._pending.size == 0))
+        if (message.pendingId == undefined /* outgoing */ ||
+            (this._app.isRoomFocused(this._room) && this._pending.size == 0))
             this._channel.ack_message_async(tpMessage, null);
         else if (this._needsIndicator)
             this._setIndicatorMark(this._view.buffer.get_end_iter());
